library card_search_bar;

import 'package:flutter/material.dart';

enum CardSearchBarState { idle, active }

///Root widget
class CardSearchBar extends StatefulWidget {
  Function(String value) onSubmit;

  CardSearchBar({this.onSubmit});

  @override
  _CardSearchBarState createState() => _CardSearchBarState(onSubmit);
}

class _CardSearchBarState extends State<CardSearchBar> {
  var _state = CardSearchBarState.idle;
  var _history = [];
  String _searchphrase = '';
  Function(String value) _onSubmit;

  _CardSearchBarState(this._onSubmit);

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(vertical: 2, horizontal: 4),
      child: Row(
        children: <Widget>[
          Expanded(
            child: Material(
              elevation: 1,
              color: Colors.white,
              borderRadius: BorderRadius.circular(4),
              child: AnimatedCrossFade(
                firstChild: _SearchIdleState(parent: this),
                secondChild: _SearchActiveState(parent: this),
                duration: Duration(milliseconds: 150),
                crossFadeState: _state == CardSearchBarState.idle
                    ? CrossFadeState.showFirst
                    : CrossFadeState.showSecond,
              ),
            ),
          ),
        ],
      ),
    );
  }
}

///Widgets set that represents IDLE state
class _SearchIdleState extends StatelessWidget {
  final _CardSearchBarState parent;

  const _SearchIdleState({Key key, @required this.parent}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () {
        parent.setState(() {
          parent._state = CardSearchBarState.active;
        });
      },
      child: Row(
        children: <Widget>[
          Expanded(
            child: Container(
              padding: EdgeInsets.only(left: 16),
              child: Text(
                parent._searchphrase != '' ? parent._searchphrase : 'Search...',
                style: TextStyle(fontSize: 18, color: Colors.grey[600]),
              ),
            ),
          ),
          IconButton(
            icon: Icon(Icons.search),
            onPressed: () {},
          ),
        ],
      ),
    );
  }
}

///Widgets set that represents ACTIVE state
class _SearchActiveState extends StatefulWidget {
  final _CardSearchBarState parent;

  const _SearchActiveState({Key key, @required this.parent}) : super(key: key);

  @override
  _SearchActiveStateState createState() => _SearchActiveStateState(parent);
}

class _SearchActiveStateState extends State<_SearchActiveState> {
  TextEditingController controller = TextEditingController();
  final _CardSearchBarState parent;

  _SearchActiveStateState(this.parent);

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Row(
          children: <Widget>[
            IconButton(
              onPressed: () {
                print('from active to idle');
                parent.setState(() {
                  parent._state = CardSearchBarState.idle;
                });
              },
              icon: Icon(Icons.arrow_back),
            ),
            Expanded(
              child: TextField(
                textInputAction: TextInputAction.search,
                decoration: InputDecoration(
                  hintStyle: TextStyle(fontSize: 18),
                  hintText: 'Search...',
                  border: InputBorder.none,
                ),
                controller: controller,
                onSubmitted: (value) {
                  if (value != null && value.length > 0) {
                    parent._onSubmit(value);
                    parent._history.add(value);
                    parent._searchphrase = value;
                    parent._state = CardSearchBarState.idle;
                  }
                  parent.setState(() {});
                },
              ),
            ),
            IconButton(
              onPressed: () {
                controller.clear();
              },
              icon: Icon(Icons.close),
            ),
          ],
        ),
        constructHistoryWidget(),
      ],
    );
  }

  Widget constructHistoryWidget() {
    List<Widget> content = new List();
    final p = parent;
    final h = p._history;

    if (h.length > 0) content.add(Divider(height: 1));
    for (int i = 1; i <= 5 && i <= h.length; i++) {
      content.add(_HistoryEntry(parent: p, historyId: h.length - i));
    }

    return Column(
      children: content,
    );
  }
}

class _HistoryEntry extends StatelessWidget {
  final _CardSearchBarState parent;
  final int historyId;

  const _HistoryEntry(
      {Key key, @required this.parent, @required this.historyId})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    print('_HistoryEntry drawn');
    return Row(
      children: <Widget>[
        IconButton(
          onPressed: () {},
          icon: Icon(Icons.history),
          color: Colors.grey[700],
        ),
        Expanded(
          child: InkWell(
            child: Container(
              height: 48.0,
              alignment: Alignment.centerLeft,
              child: Text(parent?._history[historyId]),
            ),
            onTap: () {
              parent._searchphrase = parent?._history[historyId];
              parent.setState(() {
                parent._state = CardSearchBarState.idle;
              });
            },
          ),
        ),
        IconButton(
          onPressed: () {},
          icon: Icon(Icons.close),
          color: Colors.grey[700],
        ),
      ],
    );
  }
}
